# -*- coding: utf-8 -*-
import sys
import urwid
import subprocess
import threading

from pyhn.popup import Popup
from pyhn.poller import Poller
from pyhn.config import Config
from pyhn import __version__

PY3 = False
if sys.version_info.major == 3:
    PY3 = True

if PY3:
    from urllib.parse import urlparse
else:
    from urlparse import urlparse


class ItemWidget(urwid.WidgetWrap):
    """ Widget of listbox, represent each story """
    def __init__(self, story):
        self.story = story
        self.number = story.number
        self.title = story.title.encode('utf-8')
        self.url = story.url
        self.domain = urlparse(story.domain).netloc
        self.submitter = story.submitter
        self.submitter_url = story.submitter_url
        self.comment_count = story.comment_count
        self.comments_url = story.comments_url
        self.score = story.score
        self.published_time = story.published_time

        if self.number is None:
            number_text = '-'
            number_align = 'center'
            self.number = '-'
        else:
            number_align = 'right'
            number_text = '%s:' % self.number

        if self.submitter is None:
            self.submitter = None
            self.submitter_url = None

        if self.score is None:
            self.score = "-"

        if self.comment_count is None:
            comment_text = '-'
            self.comment_count = None
            self.comments_url = None
        else:
            comment_text = '%s' % self.comment_count

        title = self.title
        try:
            title = title.encode('latin')
        except:
            pass

        self.item = [
            ('fixed', 4, urwid.Padding(urwid.AttrWrap(
                urwid.Text(number_text, align=number_align),
                'body', 'focus'))),
            urwid.AttrWrap(
                urwid.Text(title), 'body', 'focus'),
            ('fixed', 5, urwid.Padding(urwid.AttrWrap(
                urwid.Text(str(self.score), align="right"), 'body', 'focus'))),
            ('fixed', 8, urwid.Padding(urwid.AttrWrap(
                urwid.Text(comment_text, align="right"),
                'body', 'focus')))]
        w = urwid.Columns(self.item, focus_column=1, dividechars=1)
        self.__super.__init__(w)

    def selectable(self):
        return True

    def keypress(self, size, key):
        return key


class HNGui(object):
    """ The Pyhn Gui object """
    def __init__(self, cache_manager):
        self.cache_manager = cache_manager
        self.already_build = False
        self.on_comments = False
        self.which = "top"

        self.config = Config()
        self.poller = Poller(
            self, delay=int(
                self.config.parser.get('settings', 'refresh_interval')))
        self.palette = self.config.get_palette()

    def main(self):
        """
        Main Gui function which create Ui object,
        build interface and run the loop
        """
        self.ui = urwid.raw_display.Screen()
        self.ui.register_palette(self.palette)
        self.build_interface()
        self.ui.run_wrapper(self.run)

    def build_help(self):
        """ Fetch all key bindings and build help message """
        self.bindings = {}
        self.help_msg = []
        self.help_msg.append(
            urwid.AttrWrap(urwid.Text('\n Key bindings \n'), 'title'))
        self.help_msg.append(urwid.AttrWrap(urwid.Text(''), 'help'))
        for binding in self.config.parser.items('keybindings'):
            self.bindings[binding[0]] = binding[1]
            line = urwid.AttrWrap(
                urwid.Text(
                    ' %s: %s ' % (binding[1], binding[0].replace('_', ' '))),
                'help')
            self.help_msg.append(line)
        self.help_msg.append(urwid.AttrWrap(
            urwid.Text(' ctrl mouse-left: open story link'), 'help'))
        self.help_msg.append(urwid.AttrWrap(urwid.Text(''), 'help'))
        self.help_msg.append(urwid.AttrWrap(
            urwid.Text(
                ' Thanks for using Pyhn %s! ' % __version__, align='center'),
            'title'))
        self.help_msg.append(urwid.AttrWrap(urwid.Text(''), 'help'))
        self.help_msg.append(
            urwid.AttrWrap(urwid.Text(
                ' Author : toxinu'), 'help'))
        self.help_msg.append(urwid.AttrWrap(
            urwid.Text(' Code   : https://github.com/toxinu/pyhn '),
            'help'))
        self.help_msg.append(urwid.AttrWrap(
            urwid.Text(' Website: http://toxinu.github.io '),
            'help'))
        self.help_msg.append(urwid.AttrWrap(urwid.Text(''), 'help'))
        self.help_msg.append(urwid.AttrWrap(urwid.Text(''), 'help'))
        self.help_msg.append(urwid.AttrWrap(urwid.Text(''), 'help'))

        self.help = Popup(self.help_msg, ('help', 'help'), (0, 1), self.view)

    def build_interface(self):
        """
        Build interface, refresh cache if needed, update stories listbox,
        create header, footer, view and the loop.
        """
        if self.cache_manager.is_outdated():
            self.cache_manager.refresh()

        self.stories = self.cache_manager.get_stories()
        self.update_stories(self.stories)
        self.header_content = [
            ('fixed', 4, urwid.Padding(
                urwid.AttrWrap(urwid.Text(' N°'), 'header'))),
            urwid.AttrWrap(urwid.Text('TOP STORIES', align="center"), 'title'),
            ('fixed', 5, urwid.Padding(
                urwid.AttrWrap(urwid.Text('SCORE'), 'header'))),
            ('fixed', 8, urwid.Padding(
                urwid.AttrWrap(urwid.Text('COMMENTS'), 'header')))]

        self.header = urwid.Columns(self.header_content, dividechars=1)
        self.footer = urwid.AttrMap(
            urwid.Text(
                'Welcome in pyhn by toxinu '
                '(https://github.com/toxinu/pyhn)', align='center'),
            'footer')

        self.view = urwid.Frame(
            urwid.AttrWrap(
                self.listbox, 'body'), header=self.header, footer=self.footer)
        self.loop = urwid.MainLoop(
            self.view,
            self.palette,
            screen=self.ui,
            handle_mouse=True,
            unhandled_input=self.keystroke)

        self.build_help()
        self.already_build = True

    def set_help(self):
        """ Set help msg in footer """
        self.view.set_footer(
            urwid.AttrWrap(urwid.Text(self.help, align="center"), 'help'))

    def set_footer(self, msg, style="normal"):
        """ Set centered footer message """
        if style == "normal":
            self.footer = urwid.AttrWrap(urwid.Text(msg), 'footer')
            self.view.set_footer(self.footer)
        elif style == "error":
            self.footer = urwid.AttrWrap(urwid.Text(msg), 'footer-error')
            self.view.set_footer(self.footer)

    def set_header(self, msg):
        """ Set header story message """
        self.header_content[1] = urwid.AttrWrap(
            urwid.Text(msg, align="center"), 'title')
        self.view.set_header(urwid.Columns(self.header_content, dividechars=1))

    def keystroke(self, input):
        """ All key bindings are computed here """
        # QUIT
        if input in ('q', 'Q'):
            self.exit(must_raise=True)
        # LINKS
        if input in self.bindings['open_comments_link'].split(','):
            if not self.listbox.get_focus()[0].comments_url:
                self.set_footer('No comments')
            else:
                if not self.on_comments:
                    self.show_comments(self.listbox.get_focus()[0])
                    self.on_comments = True
                else:
                    self.update_stories(
                        self.cache_manager.get_stories(self.which))
                    self.on_comments = False
                self.open_webbrowser(self.listbox.get_focus()[0].comments_url)
        if input in self.bindings['show_comments_link'].split(','):
            if not self.listbox.get_focus()[0].comments_url:
                self.set_footer('No comments')
            else:
                self.set_footer(self.listbox.get_focus()[0].comments_url)
        if input in self.bindings['open_story_link'].split(','):
            self.open_webbrowser(self.listbox.get_focus()[0].url)
        if input in self.bindings['show_story_link'].split(','):
            self.set_footer(self.listbox.get_focus()[0].url)
        if input in self.bindings['open_submitter_link'].split(','):
            if not self.listbox.get_focus()[0].submitter_url:
                self.set_footer('No submitter')
            else:
                self.open_webbrowser(self.listbox.get_focus()[0].submitter_url)
        if input in self.bindings['show_submitter_link'].split(','):
            if not self.listbox.get_focus()[0].submitter_url:
                self.set_footer('No submitter')
            else:
                self.set_footer(self.listbox.get_focus()[0].submitter_url)
        # MOVEMENTS
        if input in self.bindings['down'].split(','):
            if self.listbox.focus_position - 1 in self.walker.positions():
                self.listbox.set_focus(
                    self.walker.prev_position(self.listbox.focus_position))
        if input in self.bindings['up'].split(','):
            if self.listbox.focus_position + 1 in self.walker.positions():
                self.listbox.set_focus(
                    self.walker.next_position(self.listbox.focus_position))
        if input in self.bindings['page_up'].split(','):
            self.listbox._keypress_page_up(self.ui.get_cols_rows())
        if input in self.bindings['page_down'].split(','):
            self.listbox._keypress_page_down(self.ui.get_cols_rows())
        if input in self.bindings['first_story'].split(','):
            self.listbox.set_focus(self.walker.positions()[0])
        if input in self.bindings['last_story'].split(','):
            self.listbox.set_focus(self.walker.positions()[-1])
        # STORIES
        if input in self.bindings['newest_stories'].split(','):
            self.set_footer('Syncing newest stories...')
            threading.Thread(
                None,
                self.async_refresher,
                None,
                ('newest', 'NEWEST STORIES'),
                {}).start()
        if input in self.bindings['top_stories'].split(','):
            self.set_footer('Syncing top stories...')
            threading.Thread(
                None, self.async_refresher,
                None, ('top', 'TOP STORIES'), {}).start()
        if input in self.bindings['best_stories'].split(','):
            self.set_footer('Syncing best stories...')
            threading.Thread(
                None, self.async_refresher,
                None, ('best', 'BEST STORIES'), {}).start()
        if input in self.bindings['show_stories'].split(','):
            self.set_footer('Syncing show stories...')
            threading.Thread(
                None, self.async_refresher,
                None, ('show', 'SHOW STORIES'), {}).start()
        if input in self.bindings['show_newest_stories'].split(','):
            self.set_footer('Syncing show newest stories...')
            threading.Thread(
                None,
                self.async_refresher,
                None,
                ('show_newest', 'SHOW NEWEST STORIES'),
                {}).start()
        if input in self.bindings['ask_stories'].split(','):
            self.set_footer('Syncing ask stories...')
            threading.Thread(
                None, self.async_refresher,
                None, ('ask', 'ASK STORIES'), {}).start()
        if input in self.bindings['jobs_stories'].split(','):
            self.set_footer('Syncing jobs stories...')
            threading.Thread(
                None, self.async_refresher,
                None, ('jobs', 'JOBS STORIES'), {}).start()
        # OTHERS
        if input in self.bindings['refresh'].split(','):
            self.set_footer('Refreshing new stories...')
            threading.Thread(
                None, self.async_refresher, None, (), {'force': True}).start()
        if input in self.bindings['reload_config'].split(','):
            self.reload_config()
        if input in ('h', 'H', '?'):
            keys = True
            while True:
                if keys:
                    self.ui.draw_screen(
                        self.ui.get_cols_rows(),
                        self.help.render(self.ui.get_cols_rows(), True))
                    keys = self.ui.get_input()
                    if 'h' or 'H' or '?' or 'escape' in keys:
                        break
        # MOUSE
        if len(input) > 1 and input[0] == 'ctrl mouse release':
            self.open_webbrowser(self.listbox.get_focus()[0].url)

    def async_refresher(self, which=None, header=None, force=False):
        if which is None:
            which = self.which
        if self.cache_manager.is_outdated(which) or force:
            self.cache_manager.refresh(which)
        stories = self.cache_manager.get_stories(which)
        self.update_stories(stories)
        if header is not None:
            self.set_header(header)
            self.which = which
        self.loop.draw_screen()

    def update_stories(self, stories):
        """ Reload listbox and walker with new stories """
        items = []
        item_ids = []
        for story in stories:
            if story.id is not None and story.id in item_ids:
                story.title = "- %s" % story.title
                items.append(ItemWidget(story))
            else:
                items.append(ItemWidget(story))
            item_ids.append(story.id)

        if self.already_build:
            self.walker[:] = items
            self.update()
        else:
            self.walker = urwid.SimpleListWalker(items)
            self.listbox = urwid.ListBox(self.walker)

    def show_comments(self, story):
        pass

    def open_webbrowser(self, url):
        """ Handle url and open sub process with web browser """
        if self.config.parser.get('settings', 'browser_cmd') == "__default__":
            python_bin = sys.executable
            subprocess.Popen(
                [python_bin, '-m', 'webbrowser', '-t', url],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
        else:
            cmd = self.config.parser.get('settings', 'browser_cmd')
            try:
                p = subprocess.Popen(
                    cmd.replace('__url__', url),
                    shell=True,
                    close_fds=True,
                    stderr=subprocess.PIPE)

                returncode = p.wait()
            except KeyboardInterrupt:
                stderr = "User keyboard interrupt detected!"
                self.set_footer(stderr, style="error")
                return
            if returncode > 0:
                stderr = p.communicate()[1]
                self.set_footer("%s" % stderr, style="error")

    def update(self):
        """ Update footer about focus story """
        focus = self.listbox.get_focus()[0]
        if not focus.submitter:
            msg = "submitted %s" % focus.published_time
        else:
            msg = "submitted %s by %s" % (
                focus.published_time, focus.submitter)

        self.set_footer(msg)

    def reload_config(self):
        """
        Create new Config object, reload colors, refresh cache
        if needed and redraw screen.
        """
        self.set_footer('Reloading configuration')
        self.config = Config()
        self.build_help()
        self.palette = self.config.get_palette()
        self.build_interface()
        self.loop.draw_screen()
        self.set_footer('Configuration file reloaded!')

        if self.config.parser.get(
                'settings', 'cache') != self.cache_manager.cache_path:
            self.cache_manager.cache_path = self.config.parser.get(
                'settings', 'cache')

    def exit(self, must_raise=False):
        self.poller.is_running = False
        self.poller.join()
        if must_raise:
            raise urwid.ExitMainLoop()
        urwid.ExitMainLoop()

    def run(self):
        urwid.connect_signal(self.walker, 'modified', self.update)

        try:
            self.poller.start()
            self.loop.run()
        except KeyboardInterrupt:
            self.exit()
        print('Exiting... Bye!')



"""
hn-api is a simple, ad-hoc Python API for Hacker News.
======================================================

hn-api is released under the Simplified BSD License:

Copyright (c) 2010, Scott Jackson
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY SCOTT JACKSON ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL SCOTT JACKSON OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are
those of the authors and should not be interpreted as representing official
policies, either expressed or implied, of Scott Jackson.


"""
import re
import sys

import requests
from bs4 import BeautifulSoup

PY3 = False
if sys.version_info.major == 3:
    PY3 = True

if PY3:
    from urllib.parse import urljoin
    from urllib.parse import urlparse
else:
    from urlparse import urljoin
    from urlparse import urlparse

HEADERS = {
    'User-Agent': (
        "Pyhn (Hacker news command line client) - "
        "https://github.com/toxinu/pyhn")}


class HNException(Exception):
    """
    HNException is exactly the same as a plain Python Exception.

    The HNException class exists solely so that you can identify
    errors that come from HN as opposed to from your application.
    """
    pass


class HackerNewsAPI:
    """
    The class for slicing and dicing the HTML and turning
    it into HackerNewsStory objects.
    """
    number_of_stories_on_front_page = 0

    def get_source(self, url):
        """
        Returns the HTML source code for a URL.
        """
        try:
            r = requests.get(url, headers=HEADERS)
            if r:
                return r.text
        except Exception:
            raise HNException(
                "Error getting source from " + url +
                ". Your internet connection may have something "
                "funny going on, or you could be behind a proxy.")

    def get_story_number(self, source):
        """
        Parses HTML and returns the number of a story.
        """
        bs = BeautifulSoup(source, "html.parser")
        span = bs.find('span', attrs={'class': 'rank'})
        if span.string:
            number = span.string.replace('.', '')
            return int(number)

    def get_story_url(self, source):
        """
        Gets the URL of a story.
        """
        url_start = source.find('href="') + 6
        url_end = source.find('">', url_start)
        url = source[url_start:url_end]
        # Check for "Ask HN" links.
        if url[0:4] == "item":  # "Ask HN" links start with "item".
            url = "https://news.ycombinator.com/" + url

        # Change "&amp;" to "&"
        url = url.replace("&amp;", "&")

        # Remove 'rel="nofollow' from the end of links,
        # since they were causing some bugs.
        if url[len(url) - 13:] == "rel=\"nofollow":
            url = url[:len(url) - 13]

        # Weird hack for URLs that end in '" '.
        # Consider removing later if it causes any problems.
        if url[len(url) - 2:] == "\" ":
            url = url[:len(url) - 2]
        return url

    def get_story_domain(self, source):
        """
        Gets the domain of a story.
        """
        bs = BeautifulSoup(source, "html.parser")
        url = bs.find('a').get('href')
        url_parsed = urlparse(url)
        if url_parsed.netloc:
            return url
        return urljoin('https://news.ycombinator.com', url)

    def get_story_title(self, source):
        """
        Gets the title of a story.
        """
        bs = BeautifulSoup(source, "html.parser")
        title = bs.find('td', attrs={'class': 'title'}).text
        title = title.strip()
        return title

    def get_story_score(self, source):
        """
        Gets the score of a story.
        """
        bs = BeautifulSoup(source, "html.parser")
        tags = bs.find_all('span', {'class': 'score'})
        if tags:
            score = tags[0].text.split(u'\xa0')
            if not score or not score[0].isdigit():
                score = tags[0].text.split(' ')
            if score and score[0].isdigit():
                return int(score[0])

    def get_submitter(self, source):
        """
        Gets the HN username of the person that submitted a story.
        """
        bs = BeautifulSoup(source, "html.parser")
        tags = bs.find_all('a', {'class': 'hnuser'})
        if tags:
            return tags[0].text

    def get_comment_count(self, source):
        """
        Gets the comment count of a story.
        """
        bs = BeautifulSoup(source, "html.parser")
        comments = bs.find_all('a', text=re.compile('comment'))
        if comments:
            comments = comments[0].text
            separator = u'\xc2\xa0'
            if separator in comments:
                comments = comments.split(separator)[0]
            else:
                comments = comments.split(u'\xa0')[0]
            try:
                return int(comments)
            except ValueError:
                return None

        comments = bs.find_all('a', text=re.compile('discuss'))
        if comments:
            return 0

    def get_published_time(self, source):
        """
        Gets the published time ago
        """
        p = re.compile(
            r'\d{1,}\s(minutes|minute|hours|hour|day|days)\sago', flags=re.U)

        if not PY3:
            source = source.decode('utf-8')

        results = p.search(source)
        if results:
            return results.group()

    def get_hn_id(self, source):
        """
        Gets the Hacker News ID of a story.
        """
        bs = BeautifulSoup(source, "html.parser")
        hn_id = bs.find_all('a', {'href': re.compile('item\?id=')})
        if hn_id:
            hn_id = hn_id[0].get('href')
            if hn_id:
                hn_id = hn_id.split('item?id=')[-1]
                if hn_id.isdigit():
                    return int(hn_id)

    def get_comments_url(self, source):
        """
        Gets the comment URL of a story.
        """
        return "https://news.ycombinator.com/item?id=" + str(
            self.get_hn_id(source))

    def get_stories(self, source):
        """
        Looks at source, makes stories from it, returns the stories.
        """
        """ <td align=right valign=top class="title">31.</td> """
        self.number_of_stories_on_front_page = source.count(
            'span class="rank"')

        # Create the empty stories.
        news_stories = []
        for i in range(0, self.number_of_stories_on_front_page):
            story = HackerNewsStory()
            news_stories.append(story)

        soup = BeautifulSoup(source, "html.parser")
        # Gives URLs, Domains and titles.
        story_details = soup.findAll("td", {"class": "title"})
        # Gives score, submitter, comment count and comment URL.
        story_other_details = soup.findAll("td", {"class": "subtext"})
        # Get story numbers.
        story_numbers = []
        for i in range(0, len(story_details) - 1, 2):
            # Otherwise, story_details[i] is a BeautifulSoup-defined object.
            story = str(story_details[i])
            story_number = self.get_story_number(story)
            story_numbers.append(story_number)

        story_urls = []
        story_domains = []
        story_titles = []
        story_scores = []
        story_submitters = []
        story_comment_counts = []
        story_comment_urls = []
        story_published_time = []
        story_ids = []

        # Every second cell contains a story.
        for i in range(1, len(story_details), 2):
            story = str(story_details[i])
            story_urls.append(self.get_story_url(story))
            story_domains.append(self.get_story_domain(story))
            story_titles.append(self.get_story_title(story))

        for s in story_other_details:
            story = str(s)
            story_scores.append(self.get_story_score(story))
            story_submitters.append(self.get_submitter(story))
            story_comment_counts.append(self.get_comment_count(story))
            story_comment_urls.append(self.get_comments_url(story))
            story_published_time.append(self.get_published_time(story))
            story_ids.append(self.get_hn_id(story))

        # Associate the values with our newsStories.
        for i in range(0, self.number_of_stories_on_front_page):
            news_stories[i].number = story_numbers[i]
            news_stories[i].url = story_urls[i]
            news_stories[i].domain = story_domains[i]
            news_stories[i].title = story_titles[i]
            news_stories[i].score = story_scores[i]
            news_stories[i].submitter = story_submitters[i]
            if news_stories[i].submitter:
                news_stories[i].submitter_url = (
                    "https://news.ycombinator.com/user?id={}".format(
                        story_submitters[i]))
            else:
                news_stories[i].submitter_url = None
            news_stories[i].comment_count = story_comment_counts[i]
            news_stories[i].comments_url = story_comment_urls[i]
            news_stories[i].published_time = story_published_time[i]
            news_stories[i].id = story_ids[i]

            if news_stories[i].id < 0:
                news_stories[i].url.find('item?id=') + 8
                news_stories[i].comments_url = ''
                news_stories[i].submitter = None
                news_stories[i].submitter_url = None

        return news_stories

    def get_more_link(self, source):
        soup = BeautifulSoup(source, "html.parser")
        more_a = soup.findAll("a", {"rel": "nofollow"}, text="More")
        if more_a:
            return urljoin('https://news.ycombinator.com/', more_a[0]['href'])
        return None

    # #### End of internal methods. #####

    # The following methods could be turned into one method with
    # an argument that switches which page to get stories from,
    # but I thought it would be simplest if I kept the methods
    # separate.

    def get_jobs_stories(self, extra_page=1):
        stories = []
        source_latest = self.get_source("https://news.ycombinator.com/jobs")
        stories += self.get_stories(source_latest)
        for i in range(1, extra_page + 2):
            get_more_link = self.get_more_link(source_latest)
            if not get_more_link:
                break
            source_latest = self.get_source(get_more_link)
            stories += self.get_stories(source_latest)

        return stories

    def get_ask_stories(self, extra_page=1):
        stories = []
        for i in range(1, extra_page + 2):
            source = self.number_of_stories_on_front_page(
                "https://news.ycombinator.com/ask?p=%s" % i)
            stories += self.get_stories(source)
        return stories

    def get_show_newest_stories(self, extra_page=1):
        stories = []
        source_latest = self.get_source("https://news.ycombinator.com/shownew")
        stories += self.get_stories(source_latest)
        for i in range(1, extra_page + 2):
            get_more_link = self.get_more_link(source_latest)
            if not get_more_link:
                break
            source_latest = self.get_source(get_more_link)
            stories += self.get_stories(source_latest)
        return stories

    def get_show_stories(self, extra_page=1):
        stories = []
        source_latest = self.get_source("https://news.ycombinator.com/show")
        stories += self.get_stories(source_latest)
        for i in range(1, extra_page + 2):
            get_more_link = self.get_more_link(source_latest)
            if not get_more_link:
                break
            source_latest = self.get_source(get_more_link)
            stories += self.get_stories(source_latest)
        return stories

    def get_top_stories(self, extra_page=1):
        """
        Gets the top stories from Hacker News.
        """
        stories = []
        for i in range(1, extra_page + 2):
            source = self.get_source(
                "https://news.ycombinator.com/news?p=%s" % i)
            stories += self.get_stories(source)
        return stories

    def get_newest_stories(self, extra_page=1):
        """
        Gets the newest stories from Hacker News.
        """
        stories = []
        source_latest = self.get_source("https://news.ycombinator.com/newest")
        stories += self.get_stories(source_latest)
        for i in range(1, extra_page + 2):
            get_more_link = self.get_more_link(source_latest)
            if not get_more_link:
                break
            source_latest = self.get_source(get_more_link)
            stories += self.get_stories(source_latest)
        return stories

    def get_best_stories(self, extra_page=1):
        """
        Gets the "best" stories from Hacker News.
        """
        stories = []
        for i in range(1, extra_page + 2):
            source_latest = self.get_source(
                "https://news.ycombinator.com/best?p=%s" % i)
            stories += self.get_stories(source_latest)
        return stories

    def get_page_stories(self, pageId):
        """
        Gets the pageId stories from Hacker News.
        """
        source = self.get_source(
            "https://news.ycombinator.com/x?fnid=%s" % pageId)
        stories = self.get_stories(source)
        return stories


class HackerNewsStory:
    """
    A class representing a story on Hacker News.
    """
    id = 0         # The Hacker News ID of a story.
    number = None  # What rank the story is on HN.
    title = ""     # The title of the story.
    domain = ""    # The website the story is from.
    url = ""       # The URL of the story.
    score = None   # Current score of the story.
    submitter = ""        # The person that submitted the story.
    comment_count = None  # How many comments the story has.
    comments_url = ""     # The HN link for commenting (and upmodding).
    published_time = ""   # The time sinc story was published

    def get_comments(self):
        url = (
            'http://hndroidapi.appspot.com/'
            'nestedcomments/format/json/id/%s' % self.id)
        try:
            r = requests.get(url, headers=HEADERS)
            self.comments = r.json()['items']
            return self.comments
        except Exception:
            raise HNException(
                "Error getting source from " + url +
                ". Your internet connection may have something funny "
                "going on, or you could be behind a proxy.")

    def print_details(self):
        """
        Prints details of the story.
        """
        print(str(self.number) + ": " + self.title)
        print("URL: %s" % self.url)
        print("domain: %s" % self.domain)
        print("score: " + str(self.score) + " points")
        print("submitted by: " + self.submitter)
        print("sinc %s" + self.published_time)
        print("of comments: " + str(self.comment_count))
        print("'discuss' URL: " + self.comments_url)
        print("HN ID: " + str(self.id))
        print(" ")


class HackerNewsUser:
    """
    A class representing a user on Hacker News.
    """
    # Default value. I don't think anyone really has -10000 karma.
    karma = -10000
    name = ""  # The user's HN username.
    user_page_url = ""  # The URL of the user's 'user' page.
    threads_page_url = ""  # The URL of the user's 'threads' page.

    def __init__(self, username):
        """
        Constructor for the user class.
        """
        self.name = username
        self.user_page_url = (
            "https://news.ycombinator.com/user?id=" + self.name)
        self.threads_page_url = (
            "https://news.ycombinator.com/threads?id=%s" % self.name)
        self.refresh_karma()

    def refresh_karma(self):
        """
        Gets the karma count of a user from the source of their 'user' page.
        """
        hn = HackerNewsAPI()
        source = hn.get_source(self.user_page_url)
        karma_start = source.find('<td valign=top>karma:</td><td>') + 30
        karma_end = source.find('</td>', karma_start)
        karma = source[karma_start:karma_end]
        if karma is not '':
            self.karma = int(karma)
        else:
            raise HNException("Error getting karma for user " + self.name)



# -*- coding: utf-8 -*-
from time import sleep
from threading import Thread


class Poller(Thread):
    def __init__(self, gui, delay=5):
        if delay < 1:
            delay = 1

        self.gui = gui
        self.delay = delay
        self.is_running = True
        self.counter = 0
        super(Poller, self).__init__()

    def run(self):
        while self.is_running:
            sleep(0.1)
            self.counter += 0.1
            if self.counter >= self.delay * 60:
                self.gui.async_refresher(force=True)
                self.counter = 0



# -*- coding: utf-8 -*-
import os

try:
    from configparser import SafeConfigParser
except ImportError:
    from ConfigParser import SafeConfigParser


class Config(object):
    def __init__(self, config_dir=None, config_file=None):
        self.config_dir = config_dir
        self.config_file = config_file

        if config_dir is None:
            self.config_dir = os.path.join(
                os.environ.get('HOME', './'),
                '.pyhn')
        if config_file is None:
            self.config_file = "config"

        if not os.path.exists(self.config_dir):
            os.makedirs(self.config_dir)

        self.config_path = os.path.join(self.config_dir, self.config_file)

        self.parser = SafeConfigParser()
        self.read()

    def read(self):
        self.parser.read(self.config_path)

        # Keybindings
        if not self.parser.has_section('keybindings'):
            self.parser.add_section('keybindings')

        if not self.parser.has_option('keybindings', 'page_up'):
            self.parser.set('keybindings', 'page_up', 'ctrl u')
        if not self.parser.has_option('keybindings', 'page_down'):
            self.parser.set('keybindings', 'page_down', 'ctrl d')
        if not self.parser.has_option('keybindings', 'first_story'):
            self.parser.set('keybindings', 'first_story', 'g')
        if not self.parser.has_option('keybindings', 'last_story'):
            self.parser.set('keybindings', 'last_story', 'G')
        if not self.parser.has_option('keybindings', 'up'):
            self.parser.set('keybindings', 'up', 'j')
        if not self.parser.has_option('keybindings', 'down'):
            self.parser.set('keybindings', 'down', 'k')
        if not self.parser.has_option('keybindings', 'refresh'):
            self.parser.set('keybindings', 'refresh', 'r')

        if not self.parser.has_option('keybindings', 'show_comments_link'):
            self.parser.set('keybindings', 'show_comments_link', 'c')
        if not self.parser.has_option('keybindings', 'open_comments_link'):
            self.parser.set('keybindings', 'open_comments_link', 'C')
        if not self.parser.has_option('keybindings', 'show_story_link'):
            self.parser.set('keybindings', 'show_story_link', 's')
        if not self.parser.has_option('keybindings', 'open_story_link'):
            self.parser.set('keybindings', 'open_story_link', 'S,enter')
        if not self.parser.has_option('keybindings', 'show_submitter_link'):
            self.parser.set('keybindings', 'show_submitter_link', 'u')
        if not self.parser.has_option('keybindings', 'open_submitter_link'):
            self.parser.set('keybindings', 'open_submitter_link', 'U')

        if not self.parser.has_option('keybindings', 'reload_config'):
            self.parser.set('keybindings', 'reload_config', 'ctrl R')

        if not self.parser.has_option('keybindings', 'newest_stories'):
            self.parser.set('keybindings', 'newest_stories', 'n')
        if not self.parser.has_option('keybindings', 'top_stories'):
            self.parser.set('keybindings', 'top_stories', 't')
        if not self.parser.has_option('keybindings', 'best_stories'):
            self.parser.set('keybindings', 'best_stories', 'b')
        if not self.parser.has_option('keybindings', 'show_stories'):
            self.parser.set('keybindings', 'show_stories', 'd')
        if not self.parser.has_option('keybindings', 'show_newest_stories'):
            self.parser.set('keybindings', 'show_newest_stories', 'D')
        if not self.parser.has_option('keybindings', 'ask_stories'):
            self.parser.set('keybindings', 'ask_stories', 'a')
        if not self.parser.has_option('keybindings', 'jobs_stories'):
            self.parser.set('keybindings', 'jobs_stories', 'J')
        # Paths
        if not self.parser.has_section('settings'):
            self.parser.add_section('settings')

        if not self.parser.has_option('settings', 'extra_page'):
            self.parser.set('settings', 'extra_page', '1')

        if not self.parser.has_option('settings', 'cache'):
            self.parser.set(
                'settings',
                'cache',
                os.path.join(os.environ.get('HOME', './'), '.pyhn', 'cache'))
        if not self.parser.has_option('settings', 'cache_age'):
            self.parser.set('settings', 'cache_age', "5")
        if not self.parser.has_option('settings', 'browser_cmd'):
            self.parser.set('settings', 'browser_cmd', '__default__')

        if not self.parser.has_option('settings', 'refresh_interval'):
            self.parser.set('settings', 'refresh_interval', '5')

        # Colors
        if not self.parser.has_section('colors'):
            self.parser.add_section('colors')

        if not self.parser.has_option('colors', 'body'):
            self.parser.set('colors', 'body', 'default||standout')
        if not self.parser.has_option('colors', 'focus'):
            self.parser.set('colors', 'focus', 'black||underline')
        if not self.parser.has_option('colors', 'footer'):
            self.parser.set('colors', 'footer', 'black|light gray')
        if not self.parser.has_option('colors', 'footer-error'):
            self.parser.set(
                'colors', 'footer-error', 'dark red,bold|light gray')
        if not self.parser.has_option('colors', 'header'):
            self.parser.set('colors', 'header', 'dark gray,bold|white|')
        if not self.parser.has_option('colors', 'title'):
            self.parser.set('colors', 'title', 'dark red,bold|light gray')
        if not self.parser.has_option('colors', 'help'):
            self.parser.set('colors', 'help', 'black|dark cyan|standout')

        if not os.path.exists(self.config_path):
            self.parser.write(open(self.config_path, 'w'))

    def get_palette(self):
        palette = []
        for item in self.parser.items('colors'):
            name = item[0]
            settings = item[1]
            foreground = ""
            background = ""
            monochrome = ""
            if len(settings.split('|')) == 3:
                foreground = settings.split('|')[0]
                background = settings.split('|')[1]
                monochrome = settings.split('|')[2]
            elif len(settings.split('|')) == 2:
                foreground = settings.split('|')[0]
                background = settings.split('|')[1]
            elif len(settings.split('|')) == 1:
                foreground = settings.split('|')[0]

            palette.append((name, foreground, background, monochrome))
        return palette



# -*- coding: utf-8 -*-
import urwid


class Popup(urwid.WidgetWrap):
    """
    Creates a popup menu on top of another BoxWidget.

    Attributes:

    selected -- Contains the item the user has selected by pressing <RETURN>,
                or None if nothing has been selected.
    """

    selected = None

    def __init__(self, menu_list, attr, pos, body):
        """
        menu_list -- a list of strings with the menu entries
        attr -- a tuple (background, active_item) of attributes
        pos -- a tuple (x, y), position of the menu widget
        body -- widget displayed beneath the message widget
        """

        content = [w for w in menu_list]

        # Calculate width and height of the menu widget:
        height = len(menu_list)
        width = 0
        for entry in menu_list:
            if len(entry.original_widget.text) > width:
                width = len(entry.original_widget.text)

        # Create the ListBox widget and put it on top of body:
        self._listbox = urwid.AttrWrap(urwid.ListBox(content), attr[0])
        overlay = urwid.Overlay(self._listbox, body, 'center',
                                width + 2, 'middle', height)

        urwid.WidgetWrap.__init__(self, overlay)

    def keypress(self, size, key):
        """
        <RETURN> key selects an item, other keys will be passed to
        the ListBox widget.
        """

        if key == "enter":
            (widget, foo) = self._listbox.get_focus()
            (text, foo) = widget.get_text()
            self.selected = text[1:]  # Get rid of the leading space...
        else:
            return self._listbox.keypress(size, key)



# -*- coding: utf-8 -*-
import os
import pickle
import datetime

from pyhn.config import Config
from pyhn.hnapi import HackerNewsAPI


class CacheManager(object):
    def __init__(self, cache_path=None):
        self.cache_path = cache_path
        if cache_path is None:
            self.config = Config()
            self.cache_path = self.config.parser.get('settings', 'cache')

        self.cache_age = int(self.config.parser.get('settings', 'cache_age'))
        self.extra_page = int(self.config.parser.get('settings', 'extra_page'))
        self.api = HackerNewsAPI()

        if not os.path.exists(self.cache_path):
            self.refresh()

    def is_outdated(self, which="top"):
        if not os.path.exists(self.cache_path):
            return True

        try:
            cache = pickle.load(open(self.cache_path, 'rb'))
        except:
            cache = {}
        if not cache.get(which, False):
            return True

        cache_age = datetime.datetime.today() - cache[which]['date']
        if cache_age.seconds > self.cache_age * 60:
            return True
        else:
            return False

    def refresh(self, which="top"):
        if which == "top":
            stories = self.api.get_top_stories(extra_page=self.extra_page)
        elif which == "newest":
            stories = self.api.get_newest_stories(extra_page=self.extra_page)
        elif which == "best":
            stories = self.api.get_best_stories(extra_page=self.extra_page)
        elif which == "show":
            stories = self.api.get_show_stories(extra_page=self.extra_page)
        elif which == "show_newest":
            stories = self.api.get_show_newest_stories(
                extra_page=self.extra_page)
        elif which == "ask":
            stories = self.api.get_ask_stories(extra_page=self.extra_page)
        elif which == "jobs":
            stories = self.api.get_jobs_stories(extra_page=self.extra_page)
        else:
            raise Exception(
                'Bad value: top, newest, ask, jobs,'
                'show, shownewest and best stories')

        cache = {}
        if os.path.exists(self.cache_path):
            try:
                cache = pickle.load(open(self.cache_path, 'rb'))
            except:
                pass

        cache[which] = {'stories': stories, 'date': datetime.datetime.today()}
        pickle.dump(cache, open(self.cache_path, 'wb'))

    def get_stories(self, which="top"):
        cache = []
        if os.path.exists(self.cache_path):
            try:
                cache = pickle.load(open(self.cache_path, 'rb'))
            except:
                cache = {}

        if not cache.get(which, False):
            return []
        else:
            return cache[which]['stories']



#!/usr/bin/env python
# coding: utf-8
import os
import re
import sys

from codecs import open

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

requires = [
    'requests==2.10.0',
    'beautifulsoup4==4.5.1',
    'urwid==1.3.1']

version = ''
with open('pyhn/__init__.py', 'r') as fd:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

with open('README.rst', 'r', 'utf-8') as f:
    readme = f.read()

setup(
    name='pyhn',
    version=version,
    description='Hacker News in your terminal',
    long_description=readme,
    license=open("LICENSE").read(),
    author="toxinu",
    author_email="toxinu@gmail.com",
    url='https://github.com/toxinu/pyhn/',
    keywords="python hackernews hn terminal commandline",
    packages=['pyhn'],
    scripts=['scripts/pyhn'],
    install_requires=requires,
    package_data={'': ['LICENSE', 'NOTICE']},
    package_dir={'pyhn': 'pyhn'},
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5']
)
